import { NgZone } from '@angular/core';
import { Observable } from 'rxjs';
import * as THREE from 'three';
import { Plane } from 'three/src/math/Plane';
import type { NgtInstanceState } from '../abstracts/instance';
import { NgtInstance } from '../abstracts/instance';
import { Ref } from '../ref';
import { NgtStore } from '../stores/store';
import type { AnyConstructor, AnyFunction, BooleanInput, NumberInput } from '../types';
import * as i0 from "@angular/core";
export interface NgtCommonMaterialState<TMaterialParameters extends THREE.MaterialParameters = THREE.MaterialParameters, TMaterial extends THREE.Material = THREE.Material> extends NgtInstanceState<TMaterial> {
    materialParameters: TMaterialParameters;
    alphaTest?: number;
    alphaToCoverage?: boolean;
    blendDst?: THREE.BlendingDstFactor;
    blendDstAlpha?: number;
    blendEquation?: THREE.BlendingEquation;
    blendEquationAlpha?: number;
    blending?: THREE.Blending;
    blendSrc?: THREE.BlendingSrcFactor | THREE.BlendingDstFactor;
    blendSrcAlpha?: number;
    clipIntersection?: boolean;
    clippingPlanes?: Plane[];
    clipShadows?: boolean;
    colorWrite?: boolean;
    defines?: any;
    depthFunc?: THREE.DepthModes;
    depthTest?: boolean;
    depthWrite?: boolean;
    fog?: boolean;
    name?: string;
    opacity?: number;
    polygonOffset?: boolean;
    polygonOffsetFactor?: number;
    polygonOffsetUnits?: number;
    precision?: 'highp' | 'mediump' | 'lowp' | null;
    premultipliedAlpha?: boolean;
    dithering?: boolean;
    side?: THREE.Side;
    shadowSide?: THREE.Side;
    toneMapped?: boolean;
    transparent?: boolean;
    vertexColors?: boolean;
    visible?: boolean;
    format?: THREE.PixelFormat;
    stencilWrite?: boolean;
    stencilFunc?: THREE.StencilFunc;
    stencilRef?: number;
    stencilWriteMask?: number;
    stencilFuncMask?: number;
    stencilFail?: THREE.StencilOp;
    stencilZFail?: THREE.StencilOp;
    stencilZPass?: THREE.StencilOp;
    userData?: any;
}
export declare abstract class NgtCommonMaterial<TMaterialParameters extends THREE.MaterialParameters = THREE.MaterialParameters, TMaterial extends THREE.Material = THREE.Material> extends NgtInstance<TMaterial, NgtCommonMaterialState<TMaterialParameters, TMaterial>> {
    set alphaTest(alphaTest: NumberInput);
    set alphaToCoverage(alphaToCoverage: BooleanInput);
    set blendDst(blendDst: THREE.BlendingDstFactor);
    set blendDstAlpha(blendDstAlpha: NumberInput);
    set blendEquation(blendEquation: THREE.BlendingEquation);
    set blendEquationAlpha(blendEquationAlpha: NumberInput);
    set blending(blending: THREE.Blending);
    set blendSrc(blendSrc: THREE.BlendingSrcFactor | THREE.BlendingDstFactor);
    set blendSrcAlpha(blendSrcAlpha: NumberInput);
    set clipIntersection(clipIntersection: BooleanInput);
    set clippingPlanes(clippingPlanes: Plane[]);
    set clipShadows(clipShadows: BooleanInput);
    set colorWrite(colorWrite: BooleanInput);
    set defines(defines: any);
    set depthFunc(depthFunc: THREE.DepthModes);
    set depthTest(depthTest: BooleanInput);
    set depthWrite(depthWrite: BooleanInput);
    set fog(fog: BooleanInput);
    set name(name: string);
    set opacity(opacity: NumberInput);
    set polygonOffset(polygonOffset: BooleanInput);
    set polygonOffsetFactor(polygonOffsetFactor: NumberInput);
    set polygonOffsetUnits(polygonOffsetUnits: NumberInput);
    set precision(precision: 'highp' | 'mediump' | 'lowp' | null);
    set premultipliedAlpha(premultipliedAlpha: BooleanInput);
    set dithering(dithering: BooleanInput);
    set side(side: THREE.Side);
    set shadowSide(shadowSide: THREE.Side);
    set toneMapped(toneMapped: BooleanInput);
    set transparent(transparent: BooleanInput);
    set vertexColors(vertexColors: BooleanInput);
    set visible(visible: BooleanInput);
    set format(format: THREE.PixelFormat);
    set stencilWrite(stencilWrite: BooleanInput);
    set stencilFunc(stencilFunc: THREE.StencilFunc);
    set stencilRef(stencilRef: NumberInput);
    set stencilWriteMask(stencilWriteMask: NumberInput);
    set stencilFuncMask(stencilFuncMask: NumberInput);
    set stencilFail(stencilFail: THREE.StencilOp);
    set stencilZFail(stencilZFail: THREE.StencilOp);
    set stencilZPass(stencilZPass: THREE.StencilOp);
    set userData(userData: any);
    /**
     * @deprecated Use individual inputs instead. Notice: Do not mix [parameters] and individual inputs, they will not be merged. Will be removed in next major version
     */
    set parameters(v: TMaterialParameters | undefined);
    get parameters(): TMaterialParameters | undefined;
    abstract get materialType(): AnyConstructor<TMaterial>;
    constructor(zone: NgZone, store: NgtStore, parentRef: AnyFunction<Ref<THREE.Object3D>>, parentHostRef: AnyFunction<Ref<THREE.Object3D>>);
    protected preInit(): void;
    ngOnInit(): void;
    private readonly init;
    private readonly setParameters;
    private get subParameters$();
    protected readonly parameters$: Observable<{
        alphaTest: number | undefined;
        alphaToCoverage: boolean | undefined;
        blendDst: THREE.BlendingDstFactor | undefined;
        blendDstAlpha: number | undefined;
        blendEquation: THREE.BlendingEquation | undefined;
        blendEquationAlpha: number | undefined;
        blending: THREE.Blending | undefined;
        blendSrc: THREE.BlendingDstFactor | THREE.BlendingSrcFactor | undefined;
        blendSrcAlpha: number | undefined;
        clipIntersection: boolean | undefined;
        clippingPlanes: THREE.Plane[] | undefined;
        clipShadows: boolean | undefined;
        colorWrite: boolean | undefined;
        defines: any;
        depthFunc: THREE.DepthModes | undefined;
        depthTest: boolean | undefined;
        depthWrite: boolean | undefined;
        fog: boolean | undefined;
        name: string | undefined;
        opacity: number | undefined;
        polygonOffset: boolean | undefined;
        polygonOffsetFactor: number | undefined;
        polygonOffsetUnits: number | undefined;
        precision: "highp" | "mediump" | "lowp" | null | undefined;
        premultipliedAlpha: boolean | undefined;
        dithering: boolean | undefined;
        side: THREE.Side | undefined;
        shadowSide: THREE.Side | undefined;
        toneMapped: boolean | undefined;
        transparent: boolean | undefined;
        vertexColors: boolean | undefined;
        visible: boolean | undefined;
        format: THREE.PixelFormat | undefined;
        stencilWrite: boolean | undefined;
        stencilFunc: THREE.StencilFunc | undefined;
        stencilRef: number | undefined;
        stencilWriteMask: number | undefined;
        stencilFuncMask: number | undefined;
        stencilFail: THREE.StencilOp | undefined;
        stencilZFail: THREE.StencilOp | undefined;
        stencilZPass: THREE.StencilOp | undefined;
        userData: any;
    }>;
    static ɵfac: i0.ɵɵFactoryDeclaration<NgtCommonMaterial<any, any>, [null, null, { optional: true; skipSelf: true; }, { optional: true; skipSelf: true; }]>;
    static ɵdir: i0.ɵɵDirectiveDeclaration<NgtCommonMaterial<any, any>, never, never, { "alphaTest": "alphaTest"; "alphaToCoverage": "alphaToCoverage"; "blendDst": "blendDst"; "blendDstAlpha": "blendDstAlpha"; "blendEquation": "blendEquation"; "blendEquationAlpha": "blendEquationAlpha"; "blending": "blending"; "blendSrc": "blendSrc"; "blendSrcAlpha": "blendSrcAlpha"; "clipIntersection": "clipIntersection"; "clippingPlanes": "clippingPlanes"; "clipShadows": "clipShadows"; "colorWrite": "colorWrite"; "defines": "defines"; "depthFunc": "depthFunc"; "depthTest": "depthTest"; "depthWrite": "depthWrite"; "fog": "fog"; "name": "name"; "opacity": "opacity"; "polygonOffset": "polygonOffset"; "polygonOffsetFactor": "polygonOffsetFactor"; "polygonOffsetUnits": "polygonOffsetUnits"; "precision": "precision"; "premultipliedAlpha": "premultipliedAlpha"; "dithering": "dithering"; "side": "side"; "shadowSide": "shadowSide"; "toneMapped": "toneMapped"; "transparent": "transparent"; "vertexColors": "vertexColors"; "visible": "visible"; "format": "format"; "stencilWrite": "stencilWrite"; "stencilFunc": "stencilFunc"; "stencilRef": "stencilRef"; "stencilWriteMask": "stencilWriteMask"; "stencilFuncMask": "stencilFuncMask"; "stencilFail": "stencilFail"; "stencilZFail": "stencilZFail"; "stencilZPass": "stencilZPass"; "userData": "userData"; "parameters": "parameters"; }, {}, never>;
}
